#/*------------------------------------------------------------------------------
#--  This file is a part of the libuc, microcontroler library
#--  Copyright (C) 2011, Alexis Jeandet
#--
#--  This program is free software; you can redistribute it and/or modify
#--  it under the terms of the GNU General Public License as published by
#--  the Free Software Foundation; either version 3 of the License, or
#--  (at your option) any later version.
#--
#--  This program is distributed in the hope that it will be useful,
#--  but WITHOUT ANY WARRANTY; without even the implied warranty of
#--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#--  GNU General Public License for more details.
#--
#--  You should have received a copy of the GNU General Public License
#--  along with this program; if not, write to the Free Software
#--  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#-------------------------------------------------------------------------------
#--                 Author : Alexis Jeandet
#--                   Mail : alexis.jeandet@gmail.com
#-------------------------------------------------------------------------------*/

#---------------------------------------------------------------------------------
#       GCC EXECUTABLES
#---------------------------------------------------------------------------------
LIBUC_PREFIX = arm-none-eabi-
LIBUC_CC  = $(LIBUC_PREFIX)gcc
LIBUC_CXX = $(LIBUC_PREFIX)g++
LIBUC_AR  = $(LIBUC_PREFIX)ar
LIBUC_RANLIB  = $(LIBUC_PREFIX)ranlib
LIBUC_AS  = $(LIBUC_PREFIX)as
LIBUC_LD  = $(LIBUC_PREFIX)ld
LIBUC_SIZE = $(LIBUC_PREFIX)size
LIBUC_STRIP = $(LIBUC_PREFIX)strip -s
LIBUC_READELF = $(LIBUC_PREFIX)readelf
LIBUC_OBJCOPY=$(LIBUC_PREFIX)objcopy
LIBUC_OBJDUMP=$(LIBUC_PREFIX)objdump



#---------------------------------------------------------------------------------
#       GCC FLAGS
#---------------------------------------------------------------------------------
LIBUC_FMCU              = -mcpu=cortex-m4
LIBUC_CFLAGS            = -g -O3 -mlittle-endian -mthumb $(LIBUC_FMCU)
LIBUC_CFLAGS   		+= -ffreestanding -nostdlib -std=c99 
LIBUC_LDFLAGS  		= 
LIBUC_ODFLAGS  		= -x --syms
LIBUC_CPFLAGS		= 
LIBUC_CFLAGS_WARN_ON	= -Wall
LIBUC_CFLAGS_WARN_OFF	= -w
LIBUC_CFLAGS_RELEASE	= -O2
LIBUC_CFLAGS_DEBUG	= -g
LIBUC_CFLAGS_RELEASE_WITH_DEBUGINFO += -O2 -g
LIBUC_CFLAGS_STATIC_LIB	= -fPIC
LIBUC_CFLAGS_SOFT_FPU	= -msoft-float
LIBUC_CFLAGS_HARD_FPU	=

LIBUC_CXXFLAGS          = $(LIBUC_FMCU)
LIBUC_CXXFLAGS         += -g -gdwarf-2
LIBUC_CXXFLAGS         += -Wextra -Wundef -Wcast-align -mthumb -msoft-float
LIBUC_CXXFLAGS_WARN_ON	= -Wall
LIBUC_CXXFLAGS_WARN_OFF	= -w
LIBUC_CXXFLAGS_RELEASE	= -O2
LIBUC_CXXFLAGS_DEBUG	= -g
LIBUC_CXXFLAGS_RELEASE_WITH_DEBUGINFO += -O2 -g
LIBUC_CXXFLAGS_STATIC_LIB = -fPIC
LIBUC_CXXFLAGS_SOFT_FPU	= -msoft-float
LIBUC_CXXFLAGS_HARD_FPU	=

#---------------------------------------------------------------------------------
#       LIBRARY DEFINITIONS
#---------------------------------------------------------------------------------

LIBUC_LIBS_DIR              = $(LIBUC)/lib/bin/stm32f4
LIBUC_INC_DIR              = $(LIBUC)/lib/includes/stm32f4
include $(LIBUC)/rules/common/libsrules.mk

#---------------------------------------------------------------------------------
#       BOARD SUPORT PACKAGES
#---------------------------------------------------------------------------------
LIBUC_BSP_DIR=$(LIBUC)/bsp
LIBUC_BSP_BIN_DIR= $(LIBUC_BSP_DIR)/bin
LIBUC_BSP_INC_DIR= $(LIBUC_BSP_DIR)/includes
LIBUC_BSP = -L $(LIBUC_BSP_BIN_DIR)/$(BSP) -static -lbsp
LIBUC_BSP_INC = -I $(LIBUC_BSP_INC_DIR)/$(BSP)
#---------------------------------------------------------------------------------
#       DEVICE SPECIAL FILES
#---------------------------------------------------------------------------------
LINKER_SCRIPT = $(ARCHFOLDER)/stm32_flash.ld
DEVSOURCES  += $(ARCHFOLDER)/system_stm32f4xx.c $(ARCHFOLDER)/stm32f4xx_it.c
ASMDEVSOURCES += $(ARCHFOLDER)/startup_stm32f4xx.s 
ASMDEVSOURCESND = $(notdir $(ASMDEVSOURCES))
DEVSOURCESND = $(notdir $(DEVSOURCES))
ASMDEVOBJECTFILES += $(ASMDEVSOURCESND:.s=.o)
DEVOBJECTFILES += $(DEVSOURCESND:.c=.o)
DEVOBJECTFILES := $(addprefix $(OBJDIR)/,$(DEVOBJECTFILES))
ASMDEVOBJECTFILES := $(addprefix $(OBJDIR)/,$(ASMDEVOBJECTFILES))
ASMOBJECTFILES += $(ASMDEVOBJECTFILES)
OBJECTFILES += $(DEVOBJECTFILES)
STM32F4_INCDIR = $(ARCHFOLDER)

INCLUDES = $(STM32F4_INCDIR) $(LIBUC_INCLUDES) $(LIBUC_BSP_INC)
CFLAGS = $(LIBUC_CFLAGS)
#---------------------------------------------------------------------------------
#       RULES
#---------------------------------------------------------------------------------

all: 
	@echo "stm32f4-arm-none-eabi-gcc rules"


bin: $(ASMDEVSOURCESND) $(DEVSOURCESND) $(BINDIR)/$(TARGET).elf $(BINDIR)/$(TARGET).bin $(BINDIR)/$(TARGET).hex 
	@echo "compile bin"

$(DEVSOURCESND): $(DEVSOURCES)
	cp $(DEVSOURCES) ./

$(ASMDEVSOURCESND): $(ASMDEVSOURCES)
	cp $(ASMDEVSOURCES) ./


lib: $(TARGETINSTALLPATH)/$(TARGET).a
	@echo "compile lib"


%.a: $(OBJECTFILES)
	mkdir -p $(TARGETINSTALLPATH)
	mkdir -p $(HEADERSINSTALLPATH)
	$(LIBUC_AR)  rcs $@  $(OBJECTFILES)  #$(LIBUC_AR)  rcs $(TARGETINSTALLPATH)/$@  $(OBJECTFILES)
	@if [ -z $(HEADERS) ] ; then echo "No headers to install" ; else cp -f $(HEADERS) $(HEADERSINSTALLPATH)/ ; fi



#%.o: %.c
#	$(LIBUC_CC) -O3 -I $(LPC17XX_INCDIR) $(LIBUC_INCLUDES) $(LIBUC_BSP_INC) -MM $< -MF $*.d -MP
#	$(LIBUC_CC) -O3 -I $(LPC17XX_INCDIR) $(LIBUC_INCLUDES) $(LIBUC_BSP_INC) -c $(LIBUC_CFLAGS) $< -o $@

$(OBJECTFILES) : $(OBJDIR)/%.o : %.c  $(INC_LIBS_DEPS) $(HEADERS)
	mkdir -p $(OBJDIR)
	$(LIBUC_CC) -O3 -I $(INCLUDES) -MM $< -MF $*.d -MP
	$(LIBUC_CC) -O3 -I $(INCLUDES) -c $(CFLAGS) $< -o $@


$(ASMOBJECTFILES) : $(OBJDIR)/%.o : %.s
	mkdir -p $(OBJDIR)
	$(LIBUC_CC) -I $(INCLUDES) -c $(CFLAGS) $< -o $@


%.elf: $(LINKER_SCRIPT) $(ASMOBJECTFILES) $(OBJECTFILES)
	mkdir -p $(BINDIR)
	$(LIBUC_CC) $(LIBUC_LDFLAGS) $(LIBUC_CFLAGS) -T $^ -o $@ $(LIBUC_BSP) $(LIBUC_LIBRARIES) 
	$(LIBUC_OBJDUMP) $(LIBUC_ODFLAGS) $@ > $(@:.elf=.dump)
	$(LIBUC_SIZE) $@


%.bin: %.elf $(BIN_LIBS_DEPS)
	$(LIBUC_OBJCOPY) $(LIBUC_CPFLAGS) -O binary $< $*.bin
	@echo "BIN_LIBS_DEPS"
	@echo $(BIN_LIBS_DEPS)


%.hex: %.elf
	$(LIBUC_OBJCOPY) $(LIBUC_CPFLAGS) -O ihex $< $*.hex


clean:
	rm -f *.o *.d *.bin *.hex *.dump *.map *.a *.elf
	rm -f $(OBJECTFILES)
	rm -f $(BINDIR)/*.bin
	rm -f $(BINDIR)/*.hex
	rm -f $(BINDIR)/*.dump
	rm -f $(BINDIR)/*.map
	rm -f $(BINDIR)/*.a
	rm -f $(BINDIR)/*.elf
	rm -f $(DEVSOURCESND)
	rm -f $(ASMDEVSOURCESND)
	
distclean: clean libclean
	rm -f $(TARGETINSTALLPATH)/*.bin
	rm -f $(TARGETINSTALLPATH)/*.a
	rm -f Makefile
	

libclean:
	@echo "not a library"























